/*
 *	A modification patch for the sackitem library by Kevin Forchione.
 *	This change allows the system to move multiple items into the sack if
 *	that's what is needed to pick up another object.
 */

modify thing
	doTake(actor) =
	{
		local totbulk, totweight;
	
		totbulk := addbulk(actor.contents) + self.bulk;
		totweight := addweight(actor.contents);
		if (not actor.isCarrying(self))
			totweight := totweight + self.weight + addweight(self.contents);
	
		if (totweight > actor.maxweight)
			"%Your% load is too heavy. ";
		else if (totbulk > actor.maxbulk)
		{
			if ( self.moveToSackItem( actor ) )
			{
				"<<self.takeMessage>> ";
				self.moveInto( actor );
			}
			else
			"%You're% juggling so much already. ";
		}
		else
		{
			"<<self.takeMessage>> ";
			self.moveInto(actor);
		}
	}
	moveToSackItem( actor ) = {
		local c, n := [], o, s, totbulk, packbulk, max, choice, listbulk, movelist, cont, i;
		
		/* 
		*	determine if the actor is carrying a sackItem 
		*/  	
		c := actor.contents;
		o := car( c );
		while( o ) {
			if ( isclass( o, sackItem ) ) { 
				break;
			}
			c := cdr( c );
			o := car( c );
		}
		s := o;			// sackItem value
		
		if ( s = nil )
			return nil;
		
		c := actor.contents - [ s ];	// exclude the sackItem
		/*
		*	Select items that aren't being worn or that aren't lit as 
		*	candidates for the sackItem.
		*/
		packbulk := addbulk( s.contents );
		movelist := [];
		listbulk := 0;
		o := car( c );
		while( o ) {
			if ( !o.isworn && !o.islit && (packbulk + o.bulk <= s.maxbulk) && (o.bulk <= s.bulkthresh) && (o.bulk > 0)) {
				n := n + [ o ];
			}
			c := cdr( c );
			o := car( c );
		}
		// c and n are the lists of all objects available to move into the sack
		c := n;
		cont := true;
		if (c = [])
			return nil;
		while (cont) {
			/*
			*	Find largest object and put it in movelist
			*/
			n := c;
			o := car( n );
			max := 0;
			while( o ) {
				if (o.bulk > max) {
					max := o.bulk;
					choice := o;
				}
				totbulk := addbulk( s.contents ) + o.bulk;
				n := cdr( n );
				o := car( n );
			}
			if (choice != nil) {
				movelist := movelist + choice;
				listbulk := listbulk + choice.bulk;
				c := c - choice;
				if (addbulk(actor.contents) + self.bulk - listbulk <= actor.maxbulk) {
					cont := nil;
				}
				if (c = []) {
					cont := nil;
				}
			}
		}
		
		if (movelist = [])	// No objects moved
			return nil;
		if (addbulk(actor.contents) + self.bulk - listbulk > actor.maxbulk) {
			// If we move all available objects, still not enough room in hands
			return nil;
		}
		for (i:=1; i<=length(movelist); i++) {
			movelist[i].moveInto(s);
		}
		"(putting ";
		listcontgen(movelist, 0, 0);
		" into <<s.thedesc>> to make room)\n";
		if (s.isopenable && !s.isopen) {
			s.isopen := true;
			"(opening <<s.thedesc>> first)\n";
		}
		return true;
	}
;
