/*
 *	A library for a merchant class of NPC.  This system (more or less)
 *	supports verbs for buying, selling, and trading.  A merchant actor
 *	has a stockroom property that stores the location where his 
 *	merchandise is located. This can be a display case, shelf, or even
 *	the merchant itself.
 */

// First, the modified classes

modify thing
	verIoBuyFrom(actor) =
	{
		if (!self.isMerchant)
			"You can only buy things from merchants.";
	}
	verDoBuyFrom(actor, iobj) =
	{
		if (!iobj.isMerchant)
		{
			"You can only buy things from merchants. ";
			return;
		}
		if (self.location = actor)
		{
			"Why in the world are you trying to buy something you already have? ";
			return;
		}
		if (self.cost = nil)
		{
			"\^<<iobj.thedesc>> isn't selling that. ";
			return;
		}
		if (self.location <> iobj.stockroom)
		{
			"\^<<iobj.thedesc>> already sold that. ";
			return;
		}
	}
	doPreBuy(actor) = { return 0; }
	doPostBuy(actor) = {}
	verIoSellTo(actor) =
	{
		if (!self.isMerchant)
			"You can only sell things to merchants.";
	}
	verDoSellTo(actor, iobj) =
	{
		if (!iobj.isMerchant)
		{
			"You can only sell things to merchants. ";
			return;
		}
		if (!self.isIn(actor))
		{
			"You can't sell something you don't possess. ";
			return;
		}
		if (self.cost = nil)
		{
			"<<iobj.refuseToBuy>> ";
			return;
		}
	}
	doPreSell(actor) = { return 0; }
	doPostSell(actor) = {}
	verDoBargainwith(actor) = {
		"You get no response. ";
	}
	verIoTradeWith(actor) = {
		"You can't trade with an inanimate object. ";
	}
	verDoTradeWith(actor, iobj) = {
		if (!self.isIn(actor))
			"You're not carrying <<self.thedesc>>. ";
	}
	verIoTradeTo(actor) = {
		self.verIoTradeWith(actor);
	}
	verDoTradeTo(actor, iobj) = {
		self.verDoTradeWith(actor, iobj);
	}
	verIoTradewith(actor) = {
		self.verIoTradeWith(actor);
	}
	verIoTradeFor(actor) = {
		if (location = parserGetMe())	// Reverse trade
			return;
		if (!location.isactor && (self.owner = nil)) {
			"No one is carrying <<self.thedesc>>. ";
			return;
		}
		if (self.owner != nil)
			self.owner.verIoTradeWith(actor);
		else
			location.verIoTradeWith(actor);
	}
	verDoTradeFor(actor, iobj) = {
		if (location = parserGetMe())
			return;
		if (!location.isactor && (self.owner = nil)) {	// Reverse trade
			"No one is carrying <<self.thedesc>>. ";
			return;
		}
		if (self.owner != nil)
			self.owner.verIoTradeWith(actor);
		else
			location.verIoTradeWith(actor);
	}
	ioTradeFor(actor, dobj) = {
		local downer, iowner;
		if (dobj.location.isactor)
			downer := dobj.location;
		else
			downer := dobj.owner;
		if (self.location.isactor)
			iowner := self.location;
		else
			iowner := self.owner;
		if (downer = parserGetMe())
			iowner.makeTrade(actor, dobj, self);
		else
			downer.makeTrade(actor, self, dobj);
	}
;

modify Actor
	verIoTradeWith(actor) = {
		"\^<<self.thedesc>> isn't interested. ";
	}
	verIoTradeTo(actor) = {
		self.verIoTradeWith(actor);
	}
	verDoTradewith(actor) = {
		self.verIoTradeWith(actor);
	}
	ioTradeWith(actor, dobj) = {	// actor is offering dobj to this NPC
		"[Fill in 1]";
	}
	ioTradeTo(actor, dobj) = {
		self.ioTradeWith(actor, dobj);
	}
	doTradewith(actor) = {	// actor is offering nothing to this NPC
		"You need to be more specific as to what you want to trade. ";
	}
	makeTrade(actor, offerObj, forObj) = {	// actor is offering offerObj to this NPC in exchange for forObj
		"[Fill in 3]";
	}
;

modify iVerb
	action(actor) =
	{
		inherited.action(actor);
		if (actor.money > 0)
		{
			"\bYou are currently carrying $<<actor.money>>.\n";
		}
	}
;

modify basicMe
	verDoSellTo(actor, iobj) = "You'd never make it in the prostitution business. "
	makeTrade(actor, offerObj, forObj) = {
		"Trading with yourself?";
	}
;

// Now, the new verbs

/*
 *  Buy <object> [from <merchant>]
 *
 *  Used to purchase goods from a merchant.  
 */
buyVerb: deepverb
	sdesc = "buy"
	verb = 'buy' 'purchase'
	prepDefault = fromPrep
	ioAction(fromPrep) = 'BuyFrom'
	validDo(actor, obj, seqno) =
	{
		return obj.isVisible(actor);
	}
	action(actor) = {
		askdo;
	}
;

sellVerb: deepverb
	sdesc = "sell"
	verb = 'sell' 
	prepDefault = toPrep
	ioAction(toPrep) = 'SellTo'
//	validDo(actor, obj, seqno) =
//	{
//		return obj.isIn(actor);
//	}
	action(actor) = {
		askdo;
	}
;

bargainVerb: deepverb
	sdesc = "bargain with"
	verb = 'bargain with' 'negotiate with' 'haggle with' 'bargain'
	doAction = 'Bargainwith'
	action(actor) = {
		askdo;
	}
;

tradeVerb: deepverb
	verb = 'trade' 'exchange'
	sdesc = "trade"
	prepDefault = withPrep
	ioAction(withPrep) = 'TradeWith'
	ioAction(forPrep) = 'TradeFor'
	ioAction(toPrep) = 'TradeWith'
	validDo(actor, obj, seqno) =
	{
		return obj.isVisible(actor);
	}
	validIo(actor, obj, seqno) =
	{
		return obj.isVisible(actor);
	}
;

tradewidthVerb: deepverb
	verb = 'trade with' 'exchange with'
	sdesc = "trade with"
	doAction = 'Tradewith'
	validDo(actor, obj, seqno) =
	{
		return obj.isVisible(actor);
	}
;


// Classes

class merchant: thing
	isMerchant = true
	willBuy = nil
	stockroom = nil		// Need to set to a location or container
	totalsold = 0
	purchasemessage(actor, dobj) = 
	{
		"You hand over the cash to <<self.thedesc>> and 
		<<self.fmtYou>> removes <<dobj.thedesc>> from 
		<<stockroom.thedesc>>.  ";
	}
	verifyPurchase(actor, dobj) = {
		return true;
	}
	ioBuyFrom(actor, dobj) =
	{
		if (actor.money = nil)
		{
			"You don't have any money!";
			return nil;
		}
		if (actor.money < dobj.cost * self.discount / 100)
		{
			"You can't afford it.";
			return nil;
		}
		if (!self.verifyPurchase(actor, dobj))
			return nil;
		actor.money -= dobj.cost * self.discount / 100;
		self.totalsold += dobj.cost * self.discount / 100;
		dobj.cost := nil;
		if (dobj.doPreBuy(actor) = 0) {
			"<<self.purchasemessage(actor,dobj)>>";
			switch (rand(6))
			{
				case 1: "\"Enjoy.\" "; break;
				case 2: "\"Just remember, there's no warranty on that.\" "; break;
				case 3: "\"A pleasure doing business with you.\" "; break;
				case 4: "\"I'm sure you won't be disappointed.\" "; break;
				case 5: "\"An excellent choice.\" "; break;
				case 6: "\"\^<<dobj.thedesc>> of champions, my friend.\" "; break;
			}
		}
		dobj.moveInto(actor);
		dobj.doPostBuy(actor);
	}
	verIoTradeWith(actor) = {
		local dobj := parserGetObj(PO_DOBJ);
		if (dobj = cash) {
			return;
		}
		"\^<<self.thedesc>> shakes <<self.fmtYour>> head.  \"Sorry, no deal.  Don't you have any cash?\" ";
	}
	verIoTradeTo(actor) = {
		self.verIoTradeWith(actor);
	}
        makeTrade(actor, offerObj, forObj) = {  // actor is offering offerObj to this NPC in exchange for forObj
        	if (offerObj != cash) {
			"\^<<self.thedesc>> shakes <<self.fmtYour>> head.  \"Sorry, no deal.  Don't you have any cash?\" ";
        		return;
        	}
        	self.ioBuyFrom(actor, forObj);
        }
        refuseToBuy = "\^<<self.thedesc>> doesn't appear interested. "
	verDoBargainwith(actor) = {}
	doBargainwith(actor) = {
		"Your attempts to bargain fail. ";
	}
;

cash: scenery, floatingItem, knownTopic
	sdesc = "cash"
	noun = 'cash' 'money'
	location = {
		return parserGetMe();
	}
	isThem = true
	ldesc = "Old and wrinkly.  Green and black ink on pale paper.  Supposedly from before the war. "
	smelldesc = "It smells like power. "
	verDoDrop(actor) = {
		"You don't have enough to go around wasting it. ";
	}
	doGiveTo(actor, io) = {
		"Foo";
	}
;


